function w = hmc ( w , L , eta , alpha ) 

# hmc ( w , L , eta , alpha )
#			trains a single neuron from weight vector w
#                       using global data in x,t
#                       for a number of loops L
#                       with learning rate eta; 
# 			there is weight decay alpha
#
# for the crude langevin method, we add noise of magnitude ~ sqrt(eta)
# for true langevin we also compute the energy and whether to accept
# for hmc, we loop some random number of times round the leapfrog 
#
# 	 		the final weights are returned
# also the weight history is logged to wl every dT its
# using the global variable T

	global dT ; 
	global T ; 
	global dT0 ; 
	global T0 ; 
	global logtime ;
	global accepts ; 
	global rejects ; 

	global x ;       # x is a matrix containing all the input vectors
	global t ;       # t is a vector containing all the targets
	global wl ;      # weight vector w logged into this matrix
	global a ;  # activations
	global y ;  # outputs
	global e ;  # errors
	global G ;  # error function
	global EW ; # weight energy
	global alphag ; 
	global Taumax ; 
	global grads ; # number of gradients evaluated 

	alphag = alpha ; 
	epsilon = sqrt( 2 * eta ) ; 

	grads ++ ;
	gw = gradM ( w ) ; 	# initialize gradient
	M  = findM ( w ) ;      # initialize objective function
	
	for l = 1:L                  # loop L times
		p = randn ( size(w) ) ; # initial momentum is Normal(0,1)
		H = p' * p / 2 + M ;    # evaluate H(w,p)
		
		Tau = round(rand () * Taumax) +Taumax;  # pick trajectory length
		wnew = w ; 
		gwnew = gw ; 
		for tau = 1:Tau
			p = p - epsilon * gwnew / 2 ; # make half-step in p
			wnew = wnew + epsilon * p ;   # make step in w

			gwnew = gradM ( wnew ) ;   # find new gradient 
			grads ++ ;
			p = p - epsilon * gwnew / 2 ;  # make half-step in p
		endfor

		Mnew  = findM ( wnew ) ;   # find new objective function
		Hnew = p' * p / 2 + Mnew ; # evaluate new value of H

		dH = Hnew - H ;
		if ( dH < 0 ) 
			accept = 1 ;
		elseif ( rand() < exp ( - dH ) )
			accept = 1 ; 
		else
			accept = 0 ;
		endif

		if ( accept )
			accepts ++ ;
			gw = gwnew ;
			w = wnew ; 
			M = Mnew ; 
		else
			rejects ++ ;
		endif

		# keep log of w if T is a multiple of dT
		if ( ( ( T > T0 ) && ( round((T)/dT) == ((T)/dT) ) ) ||	( ( ( T <= T0 ) && ( round((T)/dT0) == ((T)/dT0) ) ) ) )
			wl(logtime,:) = [w',T,G,EW,M,dH,accept,grads] ; # Keep log of the weightvector, time, G, E_W and M; also whether acceptedm, and number of gradients
			logtime ++ ;
		endif
		T ++ ;

	endfor

endfunction
