function w = langevin ( w , L , eta , alpha ) 

# langevin ( w , L , eta , alpha )
#			trains a single neuron from weight vector w
#                       using global data in x,t
#                       for a number of loops L
#                       with learning rate eta; 
# 			there is weight decay alpha
#
# for the crude langevin method, we add noise of magnitude ~ sqrt(eta)
# for true langevin we also compute the energy and whether to accept
#
# 	 		the final weights are returned
# also the weight history is logged to wl every dT its
# using the global variable T

	global dT ; 
	global T ; 
	global dT0 ; 
	global T0 ; 
	global logtime ;

	global x ;       # x is a matrix containing all the input vectors
	global t ;       # t is a vector containing all the targets

	global wl ;      # copies of the weight vector w are logged into this matrix

	noise = sqrt( 2 * eta ) ; 

	for l = 1:L                  # loop L times

		a = x * w  ;	     # compute activations
		y = sigmoid(a) ;     # compute outputs 
		e = t - y   ;	     # compute errors
		gw = x' * e ;	     # compute the gradient vector (x' means x-transpose)
		z = randn ( size(w) ) ; 
		w = w + eta * ( gw - alpha * w ) + noise * z ;  # make step

# keep log of w if T is a multiple of dT

		if ( ( ( T > T0 ) && ( round((T)/dT) == ((T)/dT) ) ) ||	( ( ( T <= T0 ) && ( round((T)/dT0) == ((T)/dT0) ) ) ) )
			G = - (t' * log(y) + (1-t') * log( 1-y )) ; 
			EW = w' * w / 2 ; 
			M = G + alpha * EW ; 
			wl(logtime,:) = [w',T,G,EW,M] ; # Keep log of the weightvector, time, G, E_W and M
			logtime ++ ;
		endif
		T ++ ;

	endfor

endfunction
