function  [wret,logs,wl] = langevinmc ( w , L , eta , alpha ,   lperiod ,  lperiod2 , turn1 , stri ) 

# langevinmc ( w , L , eta , alpha )
#			trains a nn from weight vector w
#                       using global data in x,t
#                       for a number of loops L
#                       with learning rate eta; 
# 			there is weight decay alpha
#
# for the crude langevin method, we add noise of magnitude ~ sqrt(eta)
# for true langevin we also compute the energy and whether to accept
#
# 	 		the final weights are returned
# also the weight history is logged to wl every dT its
# using the global variable T

  global xlist  ; global beta  ; global EW ; global ED  ; global M
  global dologgw ;
  global loggw ;

  global dT ; 	global T ; 	global logtime ; 
  global accepts  ; 	global rejects  ; 

  global x ;       # x is a matrix containing all the input vectors
  global t ;       # t is a vector containing all the targets
#  global wl ;      # weight vector w logged into this matrix
  global EW ; # weight energy
  global alphag ; 

  alphag = alpha ;  # this is either a scalar or a vector.
  epsilon = sqrt( 2 * eta ) ; 

  gw = gradM ( w ) ; 	# initialize gradient
  M  = findM ( w ) ;      # initialize objective function

				# preamble specific to 'net'
  logfile=[];
T = 0 ; 
  accept = 1 ; dH = 0.0;
      logs(logtime,:) =  [T,M,beta*ED,alphag*EW,ED,EW,dH,accept,accepts,rejects] ; # Keep log
      wl(:,logtime) = w ; 
      logtime ++ ; 
  showcurves(w,xlist,0,5,"",",xt u 2:3 w p 1 3") ;
  input("ready?") ;
T ++ ;
				# END preamble specific to 'net'
	
  for l = 1:L                  # loop L times
    p = randn ( size(w) ) ; # initial momentum is Normal(0,1)
    H = p' * p / 2 + M ;    # evaluate H(w,p)
    
    p = p - epsilon * gw / 2 ; # make half-step in p
    wnew = w + epsilon * p ;   # make step in w
    
    gwnew = gradM ( wnew ) ;   # find new gradient 
    Mnew  = findM ( wnew ) ;   # find new objective function
    
    p = p - epsilon * gwnew / 2 ;  # make half-step in p
    
    Hnew = p' * p / 2 + Mnew ; # evaluate new value of H
    
    dH = Hnew - H ;
    if ( dH < 0 ) 
      accept = 1 ;
    elseif ( rand() < exp ( - dH ) )
      accept = 1 ; 
    else
      accept = 0 ;
    endif
    
    if ( accept )
      accepts ++ ;
      gw = gwnew ;
      w = wnew ; 
      M = Mnew ; 
    else
      rejects ++ ;
    endif
    
				# keep log of w if T is a multiple of dT
  if ( ((T < turn1) && round(T/lperiod)*lperiod==T ) ||
	( (T >= turn1) && round(T/lperiod2)*lperiod2==T  ) )
      logs(logtime,:) =  [T,M,beta*ED,alphag*EW,ED,EW,dH,accept,accepts,rejects] ; # Keep log
      wl(:,logtime) = w ; 
      logtime ++ ;
      strin = sprintf ( "Langevin Monte Carlo%s (%d accepts, %d rejects)" , stri , accepts, rejects  ) ;
      settitleits(strin,T) ; 

      showcurves(w,xlist,0,5,"oldc u 2:3 w l 6,",",xt u 2:3 w p 1 3") ;
    endif
    T ++ ;
    
  endfor
  wret = w  ;
  printf ("%d accepts, %d rejects\n" , accepts, rejects ) ; 
endfunction


